@@ -11,6 +11,7 @@
 #include "wmoption.h"
 #include "WinMgr.h"
 #include "wmmgr.h"
+#include "yicon.h"
 
 class YClientContainer;
 class MiniIcon;
@@ -195,7 +196,7 @@ public:
     virtual void popupSystemMenu(YWindow *owner);
     virtual void handlePopDown(YPopupWindow *popup);
 
-    virtual void configure(const YRect &r, const bool resized);
+    virtual void configure(const YRect &r);
     
     void getNewPos(const XConfigureRequestEvent &cr,
                    int &cx, int &cy, int &cw, int &ch);
@@ -307,8 +308,8 @@ public:
     YFrameWindow *mainOwner();
 
 #ifndef LITE
-    YIcon *getClientIcon() const { return fFrameIcon; }
-    YIcon *clientIcon() const;
+    ref<YIcon> getClientIcon() const { return fFrameIcon; }
+    ref<YIcon> clientIcon() const;
 #endif
 
     void getNormalGeometryInner(int *x, int *y, int *w, int *h);
@@ -330,6 +331,7 @@ public:
     void updateLayer(bool restack = true);
     //void updateWorkspace();
     void updateLayout();
+    void performLayout();
 
     void updateMwmHints();
     void updateProperties();
@@ -398,11 +400,11 @@ public:
     }
 
 #ifndef LITE
-    virtual YIcon *getIcon() const { return clientIcon(); }
+    virtual ref<YIcon> getIcon() const { return clientIcon(); }
 #endif
 
-    virtual const char *getTitle() const { return client()->windowTitle(); }
-    virtual const char *getIconTitle() const { return client()->iconTitle(); }
+    virtual ustring getTitle() const { return client()->windowTitle(); }
+    virtual ustring getIconTitle() const { return client()->iconTitle(); }
 
     YFrameButton *getButton(char c);
     void positionButton(YFrameButton *b, int &xPos, bool onRight);
@@ -501,7 +503,7 @@ private:
     WindowListItem *fWinListItem;
 #endif
 #ifndef LITE
-    YIcon *fFrameIcon;
+    ref<YIcon> fFrameIcon;
 #endif
 
     YFrameWindow *fOwner;
@@ -531,6 +533,12 @@ private:
     int fStrutTop;
     int fStrutBottom;
 
+    int fShapeWidth;
+    int fShapeHeight;
+    int fShapeTitleY;
+    int fShapeBorderX;
+    int fShapeBorderY;
+
     bool fWmUrgency;
     bool fClientUrgency;
     bool fTypeDesktop;
@@ -544,6 +552,7 @@ private:
         waLeft,
         waRight
     };
+
     void wmArrange(int tcb, int lcr);
     void wmSnapMove(int tcb, int lcr);
     int getTopCoord(int my, YFrameWindow **w, int count);
@@ -559,6 +568,7 @@ private:
     void setSize(int, int);
     void setWindowGeometry(const YRect &r) {
         YWindow::setGeometry(r);
+        performLayout();
     }
     friend class MiniIcon;
 };
@@ -586,15 +596,13 @@ extern ref<YPixmap> titleQ[2];
 extern ref<YPixmap> menuButton[3];
 
 #ifdef CONFIG_GRADIENTS
-class YPixbuf;
-
-extern ref<YPixbuf> rgbFrameT[2][2];
-extern ref<YPixbuf> rgbFrameL[2][2];
-extern ref<YPixbuf> rgbFrameR[2][2];
-extern ref<YPixbuf> rgbFrameB[2][2];
-extern ref<YPixbuf> rgbTitleS[2];
-extern ref<YPixbuf> rgbTitleT[2];
-extern ref<YPixbuf> rgbTitleB[2];
+extern ref<YImage> rgbFrameT[2][2];
+extern ref<YImage> rgbFrameL[2][2];
+extern ref<YImage> rgbFrameR[2][2];
+extern ref<YImage> rgbFrameB[2][2];
+extern ref<YImage> rgbTitleS[2];
+extern ref<YImage> rgbTitleT[2];
+extern ref<YImage> rgbTitleB[2];
 #endif
 
 #endif
