@@ -3,6 +3,7 @@
 
 #include "ypaint.h"
 #include "ycursor.h"
+#include "yarray.h"
 
 class YPopupWindow;
 class YToolTip;
@@ -22,6 +23,14 @@ extern "C" {
 #define INIT_GRADIENT(Member, Value)
 #endif
 
+struct DesktopScreenInfo {
+    int screen_number;
+    int x_org;
+    int y_org;
+    int width;
+    int height;
+};
+
 class YWindow {
 public:
     YWindow(YWindow *aParent = 0, Window win = 0);
@@ -48,7 +57,7 @@ public:
     void setGeometry(const YRect &r);
     void setSize(int width, int height);
     void setPosition(int x, int y);
-    virtual void configure(const YRect &r, const bool resized);
+    virtual void configure(const YRect &r);
 
     virtual void paint(Graphics &g, const YRect &r);
     virtual void paintFocus(Graphics &, const YRect &) {}
@@ -74,7 +83,7 @@ public:
     virtual void handleCreateWindow(const XCreateWindowEvent &createWindow);
 #endif
     void handleMapNotify(const XMapEvent &map);
-    void handleUnmapNotify(const XUnmapEvent &unmap);
+    virtual void handleUnmapNotify(const XUnmapEvent &unmap);
     virtual void handleUnmap(const XUnmapEvent &unmap);
     virtual void handleDestroyWindow(const XDestroyWindowEvent &destroyWindow);
     virtual void handleReparentNotify(const XReparentEvent &) {}
@@ -123,7 +132,7 @@ public:
     Graphics & getGraphics();
 
 #ifdef CONFIG_GRADIENTS
-    virtual ref<YPixbuf> getGradient() const {
+    virtual ref<YImage> getGradient() const {
         return (parent() ? parent()->getGradient() : null); }
 #endif    
 
@@ -171,7 +180,7 @@ public:
     void installAccelerator(unsigned key, unsigned mod, YWindow *win);
     void removeAccelerator(unsigned key, unsigned mod, YWindow *win);
 
-    void setToolTip(const char *tip);
+    void setToolTip(const ustring &tip);
 
     void mapToGlobal(int &x, int &y);
     void mapToLocal(int &x, int &y);
@@ -291,11 +300,8 @@ public:
 
     virtual void grabKeys() {}
 
-#ifdef XINERAMA
 protected:
-    int xiHeads;
-    XineramaScreenInfo *xiInfo;
-#endif
+    YArray<DesktopScreenInfo> xiInfo;
 };
 
 extern YDesktop *desktop;
@@ -307,6 +313,7 @@ extern int shapeEventBase, shapeErrorBase;
 
 #ifdef CONFIG_XRANDR
 extern int xrandrSupported;
+extern bool xrandr12;
 extern int xrandrEventBase, xrandrErrorBase;
 #endif
 
